#include "FreeRTOS.h"

#include "i2c.h"
#include "lm75.h"

//
//
//
static U8 lm75Address = LM75_ADDRESS;
static U8 lm75LastRegister = LM75_REGISTER_TEMPERATURE;
static U8 lm75Mode = 0;

//
//
//
static i2cErr_e lm75Read8 (U8 reg, int *value)
{
  i2cErr_e r;

  lm75LastRegister = reg;
  *value = 0;

  if ((r = i2cWriteBuffer (lm75Address, (U8 *) &reg, sizeof (U8))) != I2CERR_NONE)
    return r;

  return i2cReadBuffer (lm75Address, (U8 *) value, sizeof (U8));
}

static i2cErr_e lm75Write8 (U8 reg, int value)
{
  U8 buffer [2];

  buffer [0] = lm75LastRegister = reg;
  buffer [1] = value;

  return i2cWriteBuffer (lm75Address, buffer, sizeof (buffer));
}

//
//  DANGER, WILL ROBINSON!  These read and write raw 9 bit values that represent 
//  the temperature in Celcius in 0.5 degree steps, NOT in the format the LM75
//  uses internally.
//
static i2cErr_e lm75Read16 (U8 reg, int *value)
{
  i2cErr_e r;
  U8 buffer [2];

  buffer [0] = lm75LastRegister = reg;

  if (!lm75Mode)
  {
    if ((r = i2cWriteReadBuffer (lm75Address, buffer, sizeof (U8), sizeof (buffer))) != I2CERR_NONE)
      return r;
  }
  else
  {
    if ((r = i2cWriteBuffer (lm75Address, (U8 *) &reg, sizeof (U8))) != I2CERR_NONE)
      return r;

    if ((r = i2cReadBuffer (lm75Address, buffer, sizeof (buffer))) != I2CERR_NONE)
      return r;
  }

  *value = ((buffer [0] << 8) | buffer [1]) >> 7;

  return I2CERR_NONE;
}

static i2cErr_e lm75Write16 (U8 reg, int value)
{
  U8 buffer [3];

  value <<= 7;

  buffer [0] = lm75LastRegister = reg;
  buffer [1] = value;
  buffer [2] = value >> 8;

  return i2cWriteBuffer (lm75Address, buffer, sizeof (buffer));
}

//
//
//
int lm75Init (void)
{
  return lm75ConfigWrite (0);
}

void lm75SetMode (int mode)
{
  lm75Mode = mode;
}

void lm75SetAddress (U8 address)
{
  lm75Address = address;
}

int lm75ReRead (int *value)
{
  *value = 0;

  if (lm75LastRegister == LM75_REGISTER_CONFIGURATION)
    return i2cReadBuffer (lm75Address, (U8 *) value, sizeof (U8));
  else
  {
    i2cErr_e r;
    U8 buffer [2];

    if ((r = i2cReadBuffer (lm75Address, buffer, sizeof (buffer))) != I2CERR_NONE)
      return r;

    *value = ((buffer [0] << 8) | buffer [1]) >> 7;
  }

  return I2CERR_NONE;
}

int lm75TemperatureRead (int *temp)
{
  return lm75Read16 (LM75_REGISTER_TEMPERATURE, temp);
}

int lm75ConfigRead (int *configValue)
{
  return lm75Read8 (LM75_REGISTER_CONFIGURATION, configValue);
}

int lm75ConfigWrite (int configValue)
{
  return lm75Write8 (LM75_REGISTER_CONFIGURATION, configValue);
}

int lm75THYSTRead (int *thystValue)
{
  return lm75Read16 (LM75_REGISTER_THYST, thystValue);
}

int lm75THYSTWrite (int thystValue)
{
  return lm75Write16 (LM75_REGISTER_THYST, thystValue);
}

int lm75TOSTRead (int *thystValue)
{
  return lm75Read16 (LM75_REGISTER_TOS, thystValue);
}

int lm75TOSWrite (int thystValue)
{
  return lm75Write16 (LM75_REGISTER_TOS, thystValue);
}
