#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <ctype.h>

//
// Scheduler includes
//
#include "FreeRTOS.h"
#include "args.h"

//
//
//
static portCHAR *strtrim (portCHAR *s)
{
  portCHAR *t = s + strlen (s) - 1;

  while (t >= s && *t && isspace (*t))
    *t-- = '\0';

  while (*s && isspace (*s))
    s++;

  return s;
}

//
//  ilen includes the space reserved for the \0
//
portCHAR argsGetLine (int fd __attribute__ ((unused)), portCHAR *ibuffer, int ilen)
{
  portCHAR *p;

  p = ibuffer;
  *p = '\0';

  while (1)
  {
    portCHAR c;
    
    fflush (stdout);

    if (read (fd, &c, sizeof (c)) == sizeof (c))
    {
      switch (c)
      {
        case '\n' :
        case '\r' :
          printf ("\n");
          return strlen (ibuffer);

        case '\b' :
          if (p > ibuffer)
            *--p = '\0';
          printf ("\b \b");
          break;

        case 0x15 : // CTRL-U
          while (p != ibuffer)
          {
            printf ("\b \b");
            --p;
          }
          *p = '\0';
          break;

        default : 
          if (p < ibuffer + ilen - 1 && c >= ' ' && c < 0x7f)
          { 
            *p++ = c;
            *p = '\0';
            printf ("%c", c); 
          }
          else
            printf ("%c", c); 

          break;
      }
    }
  }

  return 0;
}

//
//
//
typedef enum
{
  P_EATWHITESPACE = 0,
  P_GETCHARFIRST,
  P_GETCHAR,
  P_QUOTEDGETCHAR
}
PSTATE;

int argsParse (portCHAR *cmd, portCHAR **argv, int sizeofArgv, int *argc)
{
  int maxArgs = (sizeofArgv / sizeof (argv [0])) - 1;
  portCHAR *s = strtrim (cmd);
  PSTATE pstate = P_EATWHITESPACE;

  *argc = 0;
  memset (argv, 0, sizeofArgv);

  while (*s)
  {
    switch (pstate)
    {
      case P_EATWHITESPACE :
        {
          if (!isspace (*s))
            pstate = P_GETCHARFIRST;
          else
            s++;
        }
        break;

      case P_GETCHARFIRST :
        {
          *argv++ = s;

          if (++*argc == maxArgs)
            return 1;
          if (*s == '"')
            pstate = P_QUOTEDGETCHAR;
          else
            pstate = P_GETCHAR;

          s++;
        }
        break;

      case P_GETCHAR :
        {
          if (isspace (*s)) {
            pstate = P_EATWHITESPACE;
            *s = '\0';
          } 
          else if (*s == '"')
            pstate = P_QUOTEDGETCHAR;

          s++;
        }
        break;

      case P_QUOTEDGETCHAR :
        {
          if (*s == '"')
            pstate = P_GETCHAR;

          s++;
        }
        break;
    }
  }

  return 0;
}
